;********************************************************************
;*
;* MIDI Rekorder v2
;* "Mr. MIDI" 
;* MMC/SD-card, LCD display, 5 buttons, 2 LEDs, MIDI I/O
;*
;*

;*** Includes
.include "m8def.inc"
.include "mmc.asm"
.include "lcd.asm"

;*** Constants
.equ	CPUFRQ	=1000000
.equ	BR	=31250
.equ	BAUDRC	=1		; CPUFRQ/(16*BR)-1
.equ	SECTOR	=113
.equ	T1SCL	=0x04
.equ	T1FST	=0x03
.equ	T2SCL	=0x04
.equ	CHRLN	=24
; Midi Befehle
.EQU	GMreset	=255
.EQU	MIDIalv	=0xFE
.EQU	MIDIclk	=0xF8
.EQU	MIDIstop=0xFC
; Modi
.EQU	smPLAY	=0x01
.EQU	mPLAY	=0
.EQU	smREC	=0x02
.EQU	mREC	=1
.EQU	smFULL	=0x04
.EQU	mFULL	=2
.EQU	smEOS	=0x08
.EQU	mEOS	=3
.EQU	smMMCBS	=0x10
.EQU	mMMCBS	=4
.EQU	smMMCWR	=0x20
.EQU	mMMCWR	=5
.EQU	smPAUSE	=0x40
.EQU	mPAUSE	=6
.EQU	smFFWD	=0x80
.EQU	mFFWD	=7


;*** Addresses in SRAM
.equ	CMD	=0x0060		; MMC command (5 bytes)
.equ	BUF	=0x0200		; MMC data buffer (512 bytes)
.equ	RXBUF	=0x0100		; MIDI RxBuffer


;*** Ports & Pins
; PORT D
.equ	RxD	=0		; HW-UART
.equ	TxD	=1		; -"-
.equ	KEYs	=PIND
.equ	KEY1	=2		; Play
.equ	KEY2	=3		; Stop
.equ	KEY3	=4		; Rec
.equ	KEY4	=5		; Up
.equ	KEY5	=6		; Down
.equ	KEYMASK	=0x7C
; PORT C
.equ	LCD	=PORTC
; PORT B
.equ	LEDr	=6		; Record LED
.equ	LEDp	=7		; Play LED
.equ	LEDs	=PORTB
.equ	KEY6	=0		; FFW
.equ	KEY7	=1		; RWD
.equ	KEYs2	=PINB
.equ	KEYMSK2	=0x03


;*** REG defs
.def	null	=r1		; 0
.def	oldkey	=r2
.def	work	=r16		; General working register
.def	temp	=r17		; -"-
.def	cnt	=r18		; General counter
.def	inreg	=r3
.def	mode	=r19		; Mode of state machine
.def	t2	=r5		; Timer 1 is used as counter t0, t1
.def	p0	=r6		; Set of midi data and timing
.def	p1	=r7
.def	p2	=r8
.def	sd	=r9
.def	regs	=r10		; Saves SREG
.def	mcntl	=r11		; Bytecounter im Sektor
.def	mcnth	=r12
.def	scntl	=r13		; Sektorcounter in MMC
.def	scnth	=r14
.def	scnts	=r15
.def	dcnt	=r21
.def	cfile	=r22
.def	EEard	=r23
.def	EEdrd	=r20


;***********************************************************
;* 	INT table
;*

.ORG	0
	rjmp	RESET

.ORG	OVF1addr
	rjmp	T1_OVF

.ORG	OVF2addr
	rjmp	T2_OVF

.ORG	URXCaddr
	rjmp	UART_RXCP

.ORG	UTXCaddr
	rjmp	UART_TXCP



;***********************************************************
;* 	INT routines
;*


T2_OVF:	; Timer 2 overflow
	reti


T1_OVF:	; Timer 1 overflow
	inc 	t2
	reti


UART_RXCP: ; Received a midi event
	in	regs,	SREG
	in 	inreg, 	UDR		; fetch byte from UART data reg. 
	sbrc	mode,	mREC
	  rcall	RECMIDI
	out	SREG,	regs
	set
	reti


UART_TXCP: ; Sent a midi event
	set
	reti



;***********************************************************
;* 	The program
;*

RESET:	; Start up
	ldi 	work, 	LOW(RAMEND)	; set up stack
	out 	SPL, 	work
	ldi 	work, 	HIGH(RAMEND)
	out 	SPH, 	work					 

	clr	mode
	clr	scntl
	clr	scnth
	clr	scnts
	clr	cfile

	rcall	INIT_HW

	in	work,	KEYs
	andi	work,	KEYMASK
	in	temp,	KEYs2
	andi	temp,	KEYMSK2
	or	work,	temp
	mov	oldkey,	work


MAIN:	in	work,	KEYs		; get keys
	andi	work,	KEYMASK
	in	temp,	KEYs2
	andi	temp,	KEYMSK2
	or	work,	temp

	cp	work,	oldkey
	brne	KEY_HDL

  RK:	sbrc	mode,	mPLAY
	  rcall	PLAY1
	sbrc	mode,	mREC
	  rcall	BUF2MMC

	ldi	work,	(1<<SE)
	out	MCUCR,	work
	sbrs	mode,	mPLAY
	  sleep
	clt
	rjmp	MAIN


H_FAST:	sbrs	mode,	mFFWD
	  rjmp	RH
	cbr	mode,	smFFWD
	ldi	work,	T1SCL
	sbrc	mode,	mPLAY
	  out	TCCR1B,	work
	rjmp	RH

KEY_HDL:mov	oldkey,	work
	cpi	work,	KEYMASK|KEYMSK2
	breq	H_FAST

	sbis	PIND,	2
	  rjmp	PLAY
	sbis	PIND,	4
	  rjmp	REC
	sbis	PIND,	3
	  rjmp	STOP
	sbis	PIND,	5
	  rjmp	INCFN
	sbis	PIND,	6
	  rjmp	DECFN
	sbis	PINB,	0
	  rjmp	FFW
	sbis	PINB,	1
	  rjmp	RWD

  RH:	rcall	delay
	rjmp	RK



; *** PLAY Taste

PLAY:	sbrc	mode,	mREC
	  rjmp	RH
	sbrc	mode,	mPAUSE
	  rjmp	H_PAUSE
	sbrc	mode,	mPLAY
	  rjmp	H_PLAY

	; Modus Play
	sbr 	mode, 	smPLAY

	cbi	LEDs,	LEDp
	sbi	LEDs,	LEDr

	ldi	work,	CHRLN
	ldi	ZL,	LOW(t_play<<1)
	ldi	ZH,	HIGH(t_play<<1)
	rcall	ROM_LINE
	rcall	SHOWFILENR
	; Pointer auf song start
	rcall	INITMIDI
	; Lies ersten Datensatz
	rcall	READ_SECTOR_START
 PLAYOK:rcall	READMIDI
	mov	t2,	p2
	out	TCNT1H,	p1
	out	TCNT1L,	p0
	; Timer start
	ldi 	work, 	T1SCL
	out 	TCCR1B,	work
	rjmp	RH

 H_PLAY:sbr	mode,	smPAUSE
	out	TCCR1B,	null
	rcall	SENDALLOFF
	ldi	work,	CHRLN
	ldi	ZL,	LOW(t_pause<<1)
	ldi	ZH,	HIGH(t_pause<<1)
	rcall	ROM_LINE
	rjmp	RH

 H_PAUSE:cbr	mode,	smPAUSE
	ldi	work,	CHRLN
	ldi	ZL,	LOW(t_play<<1)
	ldi	ZH,	HIGH(t_play<<1)
	rcall	ROM_LINE
	ldi 	work, 	T1SCL
	out 	TCCR1B,	work
	rjmp	RH


; *** REC Taste

REC:	sbrc	mode,	mPLAY
	  rjmp	RH
	sbrc	mode,	mREC
	  rjmp	RH
	; Modus Record
	sbr 	mode, 	smREC
	cbi	LEDs,	LEDr
	sbi	LEDs,	LEDp
	ldi	work,	CHRLN
	ldi	ZL,	LOW(t_recsb<<1)
	ldi	ZH,	HIGH(t_recsb<<1)
	rcall	ROM_LINE
	rcall	SHOWFILENR
	; Zhler reset
	rcall	INITMIDI
	ldi	YH,	HIGH(RXBUF)
	ldi	ZH,	HIGH(RXBUF)
	clr	YL
	clr	ZL
	rcall	WRITE_SECTOR_START
RECOK:	sbr	mode,	smMMCWR
	in	work,	UDR
	sbi 	UCSRB, 	RXCIE
	rjmp	RH


; *** STOP Taste

STOP:	; Stoppt REC und PLAY
	out	TCCR1B,	NULL
	; Wenn im Rec Mode, schreibe STOPP Event und RxInt Disable
	sbrs	mode,	mREC
	  rjmp	ST_
;	ldi	ZL,	LOW(t_stop<<1)
;	ldi	ZH,	HIGH(t_stop<<1)
;	rcall	SENDSTRING

	rcall	WRITESTOP
	cbi 	UCSRB, 	RXCIE
  STP:	ldi	work,	MIDISTOP
	rcall	WCHAR
	cpi	work,	0
	brne	STP	
	cbr	mode,	smMMCWR
	rcall	WRITE_SECTOR_END
  ST_:	sbrc	mode,	mPLAY
	  rcall	READ_SECTOR_END
	sbi	MMC_Write,	MMC_Chip_Select
	clr	mode
	sbi	LEDs,	LEDr
	sbi	LEDs,	LEDp
	rcall	CALCSECTOR
	rcall	delay
	rcall	INIT_MMC
	; Midigert ausmachen
	rcall	SENDALLOFF
	ldi	work,	CHRLN
	ldi	ZL,	LOW(t_stop<<1)
	ldi	ZH,	HIGH(t_stop<<1)
	rcall	ROM_LINE

	rcall	INIT_MMC
	rjmp	RH


; *** UP Taste
INCFN:	sbrc	mode,	mREC
	  rjmp	RH
	sbrc	mode,	mPLAY
	  rjmp	RH

	inc	cfile

	mov	EEdrd,	cfile
	ldi	EEard,	1
	rcall	EEwrite

	rcall	CALCSECTOR
	rjmp	RH


; *** DOWN Taste
DECFN:	sbrc	mode,	mREC
	  rjmp	RH
	sbrc	mode,	mPLAY
	  rjmp	RH

	dec	cfile

	mov	EEdrd,	cfile
	ldi	EEard,	1
	rcall	EEwrite

	rcall	CALCSECTOR
	rjmp	RH

CALCSECTOR:; Calculates sector address for current file
	ldi	work,	128
	mul	cfile,	work
	ldi	work,	LOW(SECTOR)
	add	r0,	work
	ldi	work,	HIGH(SECTOR)
	adc	r1,	work
	mov	scntl,	r0
	mov	scnth,	r1
	clr	scnts
	rcall	SHOWFILENR

;	com	work
;	out	LEDs,	work
	clr	r0
	clr	r1
	ret


; *** FFWD Taste
FFW:	sbrc	mode,	mREC
	  rjmp	RH
	sbrc	mode,	mPLAY
	  rjmp	SPDUP

	inc	scntl
	brne	EFFW
	inc	scnth
  EFFW:	rcall	SHOWSECTOR
	rjmp	RH

 SPDUP:	sbrc	mode,	mPAUSE
	  rjmp	P_FWD
	ldi	work,	T1FST
	out	TCCR1B,	work
	sbr	mode,	smFFWD
	rjmp	RH

 P_FWD:	inc	scntl
	brne	EPF
	inc	scnth
  EPF:	rcall	SHOWSECTOR
	rjmp	RESR


; *** RWD Taste
RWD:	sbrc	mode,	mREC
	  rjmp	RH

	ldi	work,	1
	sub	scntl,	work
	brsh	ERWD
	dec	scnth
  ERWD:	rcall	SHOWSECTOR

	sbrs	mode,	mPLAY
	  rjmp	RH
	
 RESR:	
  RLS:	rcall	SENDALLOFF
	rcall	Read_Byte_MMC
	rcall	INCMCNT
	cpi	work,	0
	breq	RLS
	rcall	READ_SECTOR_END
;	sbi	MMC_Write,	MMC_Chip_Select
	rcall	delay
	rcall	READ_SECTOR_START
	rcall	SHOWSECTOR
	out	TCCR1B,	null
	rcall	READMIDI
	mov	t2,	p2
	out	TCNT1H,	p1
	out	TCNT1L,	p0
	ldi	work,	T1SCL
	out	TCCR1B,	work
	rjmp	RH


; *** PLAY Polling

PLAY1:	sbrs	mode,	mPLAY		; Check Modus Play
	  ret
	sbrc	mode,	mREC		; Check Modus Play
	  ret
	sbrc	mode,	mPAUSE		; Check Modus Pause
	  ret
	;Timer t2 check
	;rcall	compT1
	;Vergleiche t (Timerstand) mit p (Liedposition)
;	ldi	ZL,	LOW(t_playpoll<<1)
;	ldi	ZH,	HIGH(t_playpoll<<1)
;	rcall	SENDSTRING

	in	work,	TCNT1L
	in	temp,	TCNT1H
	cp	t2,	p2
	breq	NCHK1
	brlo	PEXIT
	rjmp	POSOK
 NCHK1:	cp	temp,	p1
 	breq	NCHK2
	brlo	PEXIT
	rjmp	POSOK
 NCHK2:	cp	work,	p0
	brlo	PEXIT
	; Pos. passt: Gebe Daten wieder, lade neue Daten
 POSOK:	;cbr	intf,	sUTX
	clt
	out	UDR,	sd

 WPLAY:	brtc	WPLAY
;	cbr	intf,	sUTX
	clt

	rcall	READMIDI
 PEXIT:	ret





;***********************************************************
;* 	Additional routines
;*

INIT_HW: ; Inits the micro; no params
	clr 	null

	; Timer setup
	out 	TCCR1B,	NULL		; T1SCL to start
	out	TCNT1H,	NULL
	out 	TCNT1L,	NULL		; Timer 0.5ms
	ldi	work,	T2SCL
	out	TCCR2,	work
	ldi 	work,	(1<<TOIE1)|(1<<TOIE2)
	out 	TIMSK,	work		; Timer interrupts

	; UART setup
	ldi 	work, 	BAUDRC		; baudrate constant
	out 	UBRRL, 	work		; prepare baud rate register
	sbi 	UCSRB, 	RXEN		; set "receive enable flag"
	sbi 	UCSRB, 	TXEN		; set "transmitter enable flag"
	cbi 	UCSRB, 	RXCIE		; and "received character interrupt enable flag"
	sbi 	UCSRB, 	TXCIE		; and "received character interrupt enable flag"
	sei
	in	inreg,	UDR
	clt
	
	; PORT setup
	ldi	work,	0xEC
 	out	DDRB,	work		; //PB7=SCK,PB6=MISO,PB5=MOSI,PB4=MMC_CS,PB3..0 not used
	ldi	work,	0xC7
	out	PORTB,	work
	ldi	work,	KEYMASK
	out	PORTD,	work
	out	DDRD,	null
	ldi	work,	0xFF
	out	DDRC,	work
	out	PORTC,	work

	rcall	LCD_INIT
   RTI:	rcall	LCD_CLEAR
	rcall	SENDALLOFF

	rcall	INIT_MMC
	rcall	delay
	rcall	INIT_MMC
	cpi	work,	0
	breq	COK

	; Card failed
	ldi	work,	0
	ldi	ZL,	LOW(t_initf<<1)
	ldi	ZH,	HIGH(t_initf<<1)
	rcall	ROM_LINE
	ldi	work,	CHRLN
	ldi	ZL,	LOW(t_error<<1)
	ldi	ZH,	HIGH(t_error<<1)
	rcall	ROM_LINE
	ldi	work,	31
	rcall	SET_CURSOR
	rcall	INIT_MMC
	ldi	temp,	'0'
	add	temp,	work
	rcall	LCD_DATA

   v:	sbic	KEYs,	KEY1
	  rjmp v
	rjmp	RTI

	; Card OK
  COK:	ldi	EEard,	1
	rcall	EEread
	mov	cfile,	EEdrd
	rcall	CALCSECTOR

	rcall	SHOWFILENR
	ldi	work,	CHRLN
	ldi	ZL,	LOW(t_init<<1)
	ldi	ZH,	HIGH(t_init<<1)
	rcall	ROM_LINE

	ret


SENDSTRING:; Sends a string from ROM over serial conn.
	lpm	cnt,	Z+
	out	UDR,	cnt
  ml1:	brtc	ml1
	clt
	cpi	cnt,	13
	brne	SENDSTRING
	ret


SENDCHAR:; Sends a char in work
	out	UDR,	work
  ml_:	brtc	ml_
	clt
	ret


WRITESTOP:; Schreibt Stoppevent
	ldi	work,	MIDIstop
	mov	inreg,	work
	rcall	WRITEMIDI
	rcall	BUF2MMC
	ret


SENDRESET:; Setzt MIDI Gert zurck
	ldi	work,	GMreset
	out	UDR,	work
  WFS:	brtc	WFS
	ret


SENDALLOFF:; All off
	ldi	temp,	0x7B
	rcall	SENDALLCH
	ldi	temp,	0x79
	rcall	SENDALLCH
	ret


SENDALLCH:; Sends to all channels
	clr	cnt
 SAO:	ldi	work,	0xB0
	add	work,	cnt
	rcall	SENDCHAR
	mov	work,	temp
	rcall	SENDCHAR
	ldi	work,	0x00
	rcall	SENDCHAR
	inc	cnt
	cpi	cnt,	0x10
	brlo	SAO
	ret


RECMIDI:; Zeichnet ein Midievent auf
;	sbrs	intf,	fURX	; Nur wenn ein Zeichen gekommen ist
;	  ret
	push	XL
	push	XH
	push	ZL
	push	ZH
	push	work
	push	cnt
	push	temp
;	cbr	intf,	sURX
	;Timer t2 check
	;rcall	compT1
	; Prfe Daten
	mov	work,	inreg
	cpi	work,	MIDIclk
	breq	EXR
	cpi	work,	MIDIalv
	breq	EXR
	; Anzeige
	cbi	LEDs,	LEDp
	; Timer start
	in	work,	TCCR1B
	andi	work,	7
	cpi	work,	T1SCL
	breq	RM1

	ldi	work,	CHRLN
	ldi	ZL,	LOW(t_rec<<1)
	ldi	ZH,	HIGH(t_rec<<1)
	rcall	ROM_LINE
	ldi 	work, 	T1SCL
	out 	TCCR1B,	work

	; Schreibe Daten
  RM1:	rcall	WRITEMIDI
	
  EXR:	pop	temp
	pop	cnt
	pop	work
	pop	ZH
	pop	ZL
	pop	XH
	pop	XL
	ret


READMIDI:; Liest einen Datensatz ein
;	sbrc	mode,	mMMCBS
;	  rjmp	READMIDI
;	ldi	ZL,	LOW(t_getdata<<1)
;	ldi	ZH,	HIGH(t_getdata<<1)
;	rcall	SENDSTRING

	rcall	RDCHAR
	mov	p2,	work
	rcall	RDCHAR
	mov	p1,	work
	rcall	RDCHAR
	mov	p0,	work
	rcall	RDCHAR
	mov	sd,	work
	cpi	work,	MIDIstop
	breq	H_EOS
	sbrc	mode,	mFULL		; Voll?
	  rjmp	H_FULL
	ret
 H_EOS:	; Ende vom Lied
	out	TCCR1B,	NULL
	clr	mode
	; Midigert ausmachen
	rcall	SENDALLOFF
	rcall	READ_SECTOR_END
	sbi	MMC_Write,	MMC_Chip_Select
	sbi	LEDs,	LEDp
	rcall	CALCSECTOR

	ldi	work,	CHRLN
	ldi	ZL,	LOW(t_stop<<1)
	ldi	ZH,	HIGH(t_stop<<1)
	rcall	ROM_LINE

	rcall	INIT_MMC
	rcall	delay
	rcall	INIT_MMC
	ret


WRITEMIDI:; Speichert Miditiming und Byte
	in	cnt,	TCNT1L
	in	temp,	TCNT1H
	mov 	work, 	t2
	rcall 	WRTCHAR
	mov 	work, 	temp
	rcall 	WRTCHAR
	mov 	work, 	cnt
	rcall 	WRTCHAR
	mov 	work, 	inreg
	rcall 	WRTCHAR
	sbrc	mode,	mFULL		; Voll?
	  rjmp	H_FULL
	ret
 H_FULL:; Speicher fr Song voll!
	out	TCCR1B,	NULL
	clr	mode
	; Midigert ausmachen
;	rcall	sendRESET
	
	ret


WRTCHAR:; Speichert 1 Byte <work>->BUFFER->MMC
	st	Y+,	work
	ldi	YH,	HIGH(RXBUF)
	cp	ZL,	YL
	breq	BFULL
	ret

 BFULL:	ldi	work,	CHRLN
	ldi	ZL,	LOW(t_buffull<<1)
	ldi	ZH,	HIGH(t_buffull<<1)
	rcall	ROM_LINE
	rjmp	EW


BUF2MMC:;BUFFER->MMC kopieren
	sbrs	mode,	mMMCWR
	  ret
	sbrc	mode,	mMMCBS
	  ret
	cp	ZL,	YL
	brne	HR
	ret

  HR:	ld	work,	Z+
	ldi	ZH,	HIGH(RXBUF)
	rcall	WCHAR
	cp	ZL,	YL
	brne	HR
	ret

 WCHAR:	rcall	Write_Byte_MMC
	rcall	INCMCNT
	cpi	work,	1
	ldi	work,	0
	brne	EXWC
	cbr	mode,	smMMCWR
	rcall	WRITE_SECTOR_END
  WOK:	inc	scntl
	brne	NSCW
	inc	scnth
  NSCW:	rcall	SHOWSECTOR
	rcall	WRITE_SECTOR_START
  WOK2:	sbr	mode,	smMMCWR
	ldi	work,	1
  EXWC:	ret
	

RDCHAR:; Liest 1 Byte MMC-><work>
	rcall	Read_Byte_MMC
	push	work
	rcall	INCMCNT
	cpi	work,	1
	brne	EXRC
	rcall	READ_SECTOR_END
  RDOK:	inc	scntl
	brne	NSCT
	inc	scnth
  NSCT:	rcall	READ_SECTOR_START
  RDOK2:rcall	SHOWSECTOR
  EXRC:	pop	work
	ret


INCMCNT:; Erhht den Bytezhler, work=1 falls 512 erreicht! (und Reset Zhler)
	clr	work
	inc	mcntl
	brne	EXC
	inc	mcnth
	mov	temp,	mcnth
	cpi	temp,	2
	brlo	EXC
	clr	mcnth
	clr	mcntl
	ldi	work,	1
  EXC:	ret


INITMIDI: ; Midi Rekorder/Player vorbereiten
	; Zhler reset
	clr 	t2
	cbr	mode,	smFULL
	cbr	mode,	smEOS
	; Timer 1 Reset
	out	TCCR1B,	NULL
	out	TCNT1H,	NULL
	out	TCNT1L,	NULL
	; Ldt Seiten-Adresse
;	rcall	CALCSECTOR
	clr	mcntl
	clr	mcnth
	ret



;***************************************************************************
;* 
;* EERead
;*
;* This subroutine waits until the EEPROM is ready to be programmed, then
;* reads the register variable "EEdrd" from address "EEard"
;*
;* Number of words	:6 + return
;* Number of cycles	:12 + return (if EEPROM is ready)
;* Low Registers used	:1 (EEdrd)
;* High Registers used:	:1 (EEard)
;*
;***************************************************************************


;***** Code

EERead:	; Reads byte from EEPROM
	sbic	EECR,	EEWE	;if EEWE not clear
	rjmp	EERead		;    wait more
	out	EEARH,	null
	out	EEARL,	EEard	;output address
	sbi	EECR,	EERE	;set EEPROM Read strobe
				;This instruction takes 4 clock cycles since
				;it halts the CPU for two clock cycles
	sbi	EECR,	EERE	;set EEPROM Read strobe 2nd time
				;This instruction takes 4 clock cycles since
				;it halts the CPU for two clock cycles
	in	EEdrd,	EEDR	;get data
	ret


;***************************************************************************
;* 
;* EEWrite
;*
;* This subroutine waits until the EEPROM is ready to be programmed, then
;* programs the EEPROM with register variable "EEdwr" at address "EEawr"
;*
;* Number of words	:5 + return
;* Number of cycles	:10 + return (if EEPROM is ready)
;* Low Registers used	:None
;* High Registers used:	;2 (EEdrd,EEard)
;*
;***************************************************************************

;***** Code

EEWrite:
	sbic	EECR,	EEWE	;if EEWE not clear
	rjmp	EEWrite		;    wait more
	out	EEARH,	null
	out	EEARL,	EEard	;output address
	out	EEDR,	EEdrd	;output data
	sbi	EECR,	EEMWE
	sbi	EECR,	EEWE	;set EEPROM Write strobe
				;This instruction takes 4 clock cycles since
				;it halts the CPU for two clock cycles
	ret



;***********************************************************
;* 	LCD routines
;*

SET_CURSOR: ; Sets the cursor (DD-RAM address) to 0..31
	; SET_CURSOR (<work> cursorpos 0..31)
	mov	temp,	work		; temp holds the address in DD-RAM of LCD
	cpi	work,	CHRLN		; line 1?
	brlo	SC3
	ldi	temp,	64-CHRLN	; set address for line 1
	add	temp,	work

  SC3:	ori	temp,	0b10000000	; set command bit
	rcall	LCD_COMMAND		; send command "SET DD-RAM ADDRESS"
	ret


LINE_OUT: ; Writes a line to the LCD
	; LINE_OUT (<SRAM @Z> *text)

  LO1:	ldi	cnt,	CHRLN		; write 19 chars
  LO2:	ld	temp,	Z+		; load char from SRAM
	rcall	LCD_DATA		; write it
	dec	cnt			; dec cnt
	brne	LO2			; until cnt = 0
	ret


NUMBER_OUT: ; Writes a decimal number 0..256 at DD-RAM pos
	; NUMBER_OUT (<work> number)
	mov	temp,	work
	clr	cnt

  NO0:	cpi	work,	100		; count 100s
	brlo	NO1
	subi	work,	100
	inc	cnt
	rjmp	NO0

  NO1:	push	cnt			; save 100s
  	tst	cnt			; write ' ' if no 100s
	brne	NO2
	ldi	cnt,	' '-'0'
  NO2:	ldi	temp,	'0'		; write 100s
	add	temp,	cnt
	push	work
	rcall	LCD_DATA
	pop	work

	clr	cnt			; count 10s
  NO3:	cpi	work,	10
	brlo	NO4
	subi	work,	10
	inc	cnt
	rjmp	NO3

  NO4:	pop	temp
	tst	cnt			; write ' ' if no 10s and no 100s
	brne	NO5
	tst	temp
	brne	NO5
	ldi	cnt,	' '-'0'
  NO5:	ldi	temp,	'0'		; write 10s
	add	temp,	cnt
	push	work
	rcall	LCD_DATA
	pop	work

	ldi	temp,	'0'		; write 1s
	add	temp,	work
	rcall	LCD_DATA

	ret


HEX_OUT:; Writes hex number to LCD
	push	work
	swap	work
	rcall	nibble2hex
	mov	temp,	work
	rcall	LCD_DATA
	pop	work
	rcall	nibble2hex
	mov	temp,	work
	rcall	LCD_DATA
	ldi	temp,	' '
	rcall	LCD_DATA
	ret


ROM_LINE: ; Writes a line from ROM
	; ROM_LINE (<work> cursorposition, <ROM @Z> *Text)
	rcall	SET_CURSOR

	ldi	cnt,	CHRLN		; write 24 chars
  RL1:	lpm	temp,	Z+		; load char from ROM
	rcall	LCD_DATA		; write it
	dec	cnt			; dec cnt
	brne	RL1			; until cnt = 0

	ret


nibble2hex:; work bits 0..3 are converted in "0".."F"
	andi	work,	0x0F
	cpi	work,	10
	brlo	NH1
	subi	work,	249
  NH1:	subi	work,	208
	ret


SHOWFILENR:
	ldi	work,	0
	ldi	ZL,	LOW(t_file<<1)
	ldi	ZH,	HIGH(t_file<<1)
	rcall	ROM_LINE
	ldi	work,	6
	rcall	SET_CURSOR
	mov	work,	cfile
	rcall	NUMBER_OUT
	ldi	temp,	' '
	rcall	LCD_DATA
	ldi	temp,	'-'
	rcall	LCD_DATA
SHOWSECTOR:
	ldi	work,	12
	rcall	SET_CURSOR
	ldi	temp,	'S'
	rcall	LCD_DATA
	ldi	temp,	' '
	rcall	LCD_DATA
	mov	work,	scnts
	rcall	HEX_OUT
	mov	work,	scnth
	rcall	HEX_OUT
	mov	work,	scntl
	rcall	HEX_OUT
	ret


READERROR:
	sbi	LEDs,	LEDp
	cbr	mode,	smPLAY
	ldi	work,	CHRLN
	ldi	ZL,	LOW(t_readerr<<1)
	ldi	ZH,	HIGH(t_readerr<<1)
	rcall	ROM_LINE
	rjmp	EW


WRITEERROR:
	sbi	LEDs,	LEDr
	cbr	mode,	smREC
	ldi	work,	CHRLN
	ldi	ZL,	LOW(t_writeerr<<1)
	ldi	ZH,	HIGH(t_writeerr<<1)
	rcall	ROM_LINE
  EW:	sbic	KEYs,	KEY1
	  rjmp	EW
	rjmp	RESET


t_init:
.DB	"Mr.MIDI by Simon-INIT OK"

t_initf:
.DB	"Mr. MIDI - CARD FAILED  "

t_error:
.DB	"ERROR #  - PLAY TO RETRY"

t_play:
.DB	"Playing...              "

t_pause:
.DB	"Paused.                 "

t_rec:
.DB	"Recording...            "

t_recsb:
.DB	"Ready To Record         "

t_stop:
.DB	"Stopped.                "

t_file:
.DB	"FILE #                  "

t_readerr:
.DB	"ERROR WHEN READING CARD!"

t_writeerr:
.DB	"ERROR WHEN WRITING CARD!"

t_buffull:
.DB	"INPUT BUFFER RAN FULL!  "
